@@ -1,10 +1,10 @@
 /**
  * Utilities for the quality of service module mod_qos.
  *
- * See http://sourceforge.net/projects/mod-qos/ for further
+ * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007 Pascal Buchbinder
+ * Copyright (C) 2007-2011 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,14 +23,14 @@
  *
  */
 
-static const char revision[] = "$Id: qs_util.c,v 1.1 2010/08/17 18:32:34 pbuchbinder Exp $";
+static const char revision[] = "$Id: qs_util.c,v 1.5 2011/01/01 20:52:04 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <config.h>
+//#include <config.h>
 
 #include "qs_util.h"
 
