@@ -2,10 +2,10 @@
  * Filter utilities for the quality of service module mod_qos
  * used to create white list rules for request line filters.
  *
- * See http://sourceforge.net/projects/mod-qos/ for further
+ * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2010 Pascal Buchbinder
+ * Copyright (C) 2007-2011 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,8 +27,8 @@
  *
  */
 
-static const char revision[] = "$Id: qsfilter2.c,v 1.3 2010/08/23 19:38:28 pbuchbinder Exp $";
-static const char g_revision[] = "9.25";
+static const char revision[] = "$Id: qsfilter2.c,v 1.46 2011/05/25 06:59:42 pbuchbinder Exp $";
+static const char g_revision[] = "9.57";
 
 /* system */
 #include <stdio.h>
@@ -38,7 +38,7 @@ static const char g_revision[] = "9.25";
 #include <unistd.h>
 #include <time.h>
 
-#include <config.h>
+//#include <config.h>
 
 /* apr */
 #include <pcre.h>
@@ -60,7 +60,7 @@ static const char g_revision[] = "9.25";
 //#include <ap_config.h>
 
 /* OpenSSL  */
-#include <openssl/stack.h>
+#include <openssl/safestack.h>
 
 #define MAX_LINE 32768
 /* 2mb */
@@ -73,7 +73,10 @@ typedef enum  {
   QS_UT_QUERY
 } qs_url_type_e;
 
-/* reserved (to be escaped): {}[]()^$.|*+?\ */
+#define QS_PCRE_RESERVED      "{}[]()^$.|*+?\\-"
+//#define QS_PCRE_RESERVED      "{}[]()^$.|*+?\"'\\-"
+
+/* reserved (to be escaped): {}[]()^$.|*+?\- */
 #define QS_UNRESERVED         "a-zA-Z0-9-\\._~% "
 #define QS_GEN                ":/\\?#\\[\\]@"
 #define QS_SUB                "!$&'\\(\\)\\*\\+,;="
@@ -85,6 +88,9 @@ typedef enum  {
 
 #define QS_OVECCOUNT 3
 
+/* request line detection */
+#define QOSC_REQ          "(OPTIONS|GET|HEAD|POST|PUT|DELETE|TRACE|CONNECT|PROPFIND|PROPPATCH|MKCOL|COPY|MOVE|LOCK|UNLOCK|VERSION-CONTROL|REPORT|CHECKOUT|CHECKIN|UNCHECKOUT|MKWORKSPACE|UPDATE|LABEL|MERGE|BASELINE-CONTROL|MKACTIVITY|ORDERPATCH|ACL|PATCH|SEARCH|BCOPY|BDELETE|BMOVE|BPROPFIND|BPROPPATCH|NOTIFY|POLL|SUBSCRIBE|UNSUBSCRIBE|X-MS-ENUMATTS|RPC_IN_DATA|RPC_OUT_DATA) /[\x20-\x21\x23-\xFF]* HTTP/"
+
 pcre *pcre_b64;
 pcre *pcre_hx;
 pcre *pcre_simple_path;
@@ -107,6 +113,10 @@ static int m_query_single_pcre = 0;
 static int m_query_len_pcre = 10;
 static int m_exit_on_error = 0;
 static int m_handler = 0;
+static pcre *m_req_regex = NULL;
+static int m_log_req_regex = 0;
+static const char *m_pfx = NULL;
+static const char *m_filter = NULL;
 
 typedef struct {
   pcre *pcre;
@@ -157,6 +167,40 @@ static char *qos_detect_b64(char *line, int silent) {
   return NULL;
 }
 
+/* escape double quotes and backslash (to be used for Apache directive) */
+static char *qs_apache_escape(apr_pool_t *pool, const char *line) {
+  char *ret = apr_pcalloc(pool, strlen(line) * 4);
+  int i = 0;
+  const char *in = line;
+  while(in && in[0]) {
+    if(in[0] == '"') {
+      ret[i] = '\\';
+      i++;
+      ret[i] = 'x';
+      i++;
+      ret[i] = '2';
+      i++;
+      ret[i] = '2';
+      i++;
+    } else if(in[0] == '\\' && in[1] == '\\') {
+      ret[i] = '\\';
+      i++;
+      ret[i] = 'x';
+      i++;
+      ret[i] = '5';
+      i++;
+      ret[i] = 'c';
+      i++;
+      in++;
+    } else {
+      ret[i] = (char)in[0];
+      i++;
+    }
+    in++;
+  }
+  return ret;
+}
+
 /* escape a string in order to be used withn a pcre */
 static char *qos_escape_pcre(apr_pool_t *pool, char *line) {
   int i = 0;
@@ -166,12 +210,12 @@ static char *qos_escape_pcre(apr_pool_t *pool, char *line) {
   int reti = 0;
   if(strlen(line) == 0) return "";
   while(in[i]) {
-    if(strchr("{}[]()^$.|*+?\"'\\-", in[i]) != NULL) {
+    if(strchr(QS_PCRE_RESERVED, in[i]) != NULL) {
       if(prev && (prev == '\\')) {
 	/* already escaped */
 	ret[reti] = in[i];
 	reti++;
-      } else if(prev && (in[i] == '\\') && (strchr("{}[]()^$.|*+?\"'\\-", in[i+1]) != NULL)) {
+      } else if(prev && (in[i] == '\\') && (strchr(QS_PCRE_RESERVED, in[i+1]) != NULL)) {
 	/* escape char */
 	ret[reti] = in[i];
 	reti++;
@@ -288,6 +332,7 @@ static void qos_init_pcre() {
   sprintf(buf, "%s{%d,}", QS_HX, m_base64);
   pcre_hx = qos_pcre_compile(buf, 0);
   pcre_simple_path = qos_pcre_compile("^"QS_SIMPLE_PATH_PCRE"$", 0);
+  m_req_regex = qos_pcre_compile(QOSC_REQ, 0);
 }
 
 static void usage(char *cmd) {
@@ -296,10 +341,11 @@ static void usage(char *cmd) {
   space[strlen(cmd)] = '\0';
   printf("\n");
   printf("Utility to generate mod_qos request line rules out from\n");
-  printf("existing access log data.\n");
+  printf("existing access/audit log data.\n");
   printf("\n");
   printf("Usage: %s -i <path> [-c <path>] [-d <num>] [-h] [-b <num>]\n", cmd);
-  printf("       %s [-p|-s|-m|-o] [-l <len>] [-n] [-e] [-u 'uni'] [-t] [-v 0|1|2]\n", space);
+  printf("       %s [-p|-s|-m|-o] [-l <len>] [-n] [-e] [-u 'uni']\n", space);
+  printf("       %s [-k <prefix>] [-t] [-f <path>] [-v 0|1|2]\n", space);
   printf("\n");
   printf("Summary\n");
   printf(" mod_qos implements a request filter which validates each request\n");
@@ -313,11 +359,11 @@ static void usage(char *cmd) {
   printf(" These directives are used to define allowed request line patterns.\n");
   printf(" Request which do not match any of thses patterns are not allowed\n");
   printf(" to access the server.\n");
-  printf(" %s is an access log analyzer used to generate filter\n", cmd);
+  printf(" %s is an audit log analyzer used to generate filter\n", cmd);
   printf(" rules (perl compatible regular expressions) which may be used\n");
-  printf(" with mod_qos to deny access for suspect request lines\n");
-  printf(" (QS_PermitUri). It parses existing access log files in order to\n");
-  printf(" generate request line patterns cover all allowed request URIs.\n");
+  printf(" by mod_qos to deny access for suspect requests (QS_PermitUri rules).\n");
+  printf(" It parses existing audit log files in order to generate request\n");
+  printf(" patterns covering all allowed requests.\n");
   printf("\n");
   printf("Options\n");
   printf("  -i <path>\n");
@@ -381,12 +427,21 @@ static void usage(char *cmd) {
   printf("  -e\n");
   printf("     Exit on error.\n");
   printf("  -u 'uni'\n");
-  printf("     Use the same settings as you have used for the QS_Decoding directive.\n");
+  printf("     Enables additional decoding methods. Use the same settings as you have\n");
+  printf("     used for the QS_Decoding directive.\n");
   printf("  -p\n");
   printf("     Repesents query by pcre only (no literal strings).\n");
   printf("     Determines the worst case performance for the generated whitelist\n");
   printf("     by applying each rule for each request line (output is real time\n");
   printf("     filter duration per request line in milliseconds).\n");
+  printf("  -k <prefix>\n");
+  printf("     Prefix used to generate rule identifiers (QSF by default).\n");
+  printf("  -t\n");
+  printf("     Calculates the maximal latency per request (worst case) using the\n");
+  printf("     generated rules.\n");
+  printf("  -f <path>\n");
+  printf("     Filters the input by the provided path (prefix) only processing\n");
+  printf("     matching lines.\n");
   printf("  -v <level>\n");
   printf("     Verbose mode. (0=silent, 1=rule source, 2=detailed). Default is 1.\n");
   printf("     Don't use rules you haven't checked the request data used to\n");
@@ -426,7 +481,7 @@ static void usage(char *cmd) {
   printf("  QS_PermitUri +QSF003 deny \"^(/[a-zA-Z0-9\\-_]+)+[/]?\\.?[a-zA-Z]{0,4}$\"\n");
   printf("\n");
   printf("mod_qos %s\n", g_revision);
-  printf("See http://mod-qos.sourceforge.net/ for further details.\n");
+  printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
   exit(1);
 }
 
@@ -516,8 +571,15 @@ static char *qos_2pcre(apr_pool_t *pool, const char *line) {
 	strcpy(&ret[reti], "\\-");
 	reti = reti + 2;
       }
+    } else if(in[i] == '\0') {
+      char *ck = apr_psprintf(pool, "#\\x%02x#", in[i]);
+      if(strstr(existing, ck) == NULL) {
+        sprintf(&ret[reti], "\\x%02x", in[i]);
+        reti = reti + 4;
+        existing = apr_pstrcat(pool, existing, ck, NULL);
+      }
     } else if(strchr(ret, in[i]) == NULL) {
-      if(strchr("{}[]()^$.|*+?\"'", in[i]) != NULL) {
+      if(strchr(QS_PCRE_RESERVED, in[i]) != NULL) {
 	ret[reti] = '\\';
 	reti++;
 	ret[reti] = in[i];
@@ -971,13 +1033,13 @@ static char *qos_query_string_pcre(apr_pool_t *pool, const char *path) {
     return ret;
   } else {
     return ret;
-    /* it woud be nice to use:
+    /* it woud be nice to use (see -o):
      *  ((a=b)?(c=d)?)* 
      * instead of:
      *  (a=b)?(c=d)? and (c=d)?(a=b)?
      * but in this case, two rules are much faster than one
-     * it's better to use the -m option */
-    //return apr_psprintf(pool, "(%s)*", ret);
+     * it's probably better to use the -m option
+     */
   }
 }
 
@@ -1194,24 +1256,70 @@ static char *qos_post_optimization(apr_pool_t *lpool, char *query) {
   return query;
 }
 
+static void qos_auto_detect(char **raw) {
+  char *line = *raw;
+  int rc_c = -1;
+  if(m_req_regex) {
+    int ovector[QS_OVECCOUNT];
+    /* no request line, maybe raw Apache access log? */
+    rc_c = pcre_exec(m_req_regex, NULL, line, strlen(line), 0, 0, ovector, QS_OVECCOUNT);
+    if(rc_c >= 0) {
+      char *sr;
+      line = &line[ovector[0]];
+      line[ovector[1] - ovector[0]] = '\0';
+      sr = strchr(line, ' ');
+      while(sr[0] == ' ')sr++;
+      *raw = sr;
+      sr = strrchr(line, ' ');
+      sr[0] = '\0';
+    }
+  }
+  if(rc_c < 0) {
+    /* or an audit log like "%h %>s %{qos-loc}n %{qos-path}n%{qos-query}n" */
+    char *pe = line;
+    int pi = 3;
+    while(pe && (pi > 0)) {
+      pi--;
+      pe = strchr(pe, ' ');
+      if(pe) {
+	pe++;
+      }
+    }
+    if(pe && pe[0] == '/' && (pi == 0)) {
+      *raw = pe;
+    }
+  }
+  return;
+}
+
 /* process the input file line by line */
 static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_t *rules,
 			    apr_table_t *rules_url, apr_table_t *special_rules,
 			    FILE *f, int *ln, int *dc, int first) {
-  char *line = apr_pcalloc(pool, MAX_LINE_BUFFER);
+  char *readline = apr_pcalloc(pool, MAX_LINE_BUFFER);
   int deny_count = *dc;
   int line_nr = *ln;
   apr_table_t *source_rules = apr_table_make(pool, 10);
   int rule_optimization = 300;
-  while(!qos_fgetline(line, MAX_LINE_BUFFER, f)) {
+  while(!qos_fgetline(readline, MAX_LINE_BUFFER, f)) {
     int doubleSlash = 0;
     apr_uri_t parsed_uri;
     apr_pool_t *lpool;
+    char *line = readline;
     apr_pool_create(&lpool, NULL);
     line_nr++;
     if((strlen(line) > 1) && line[1] == '/') {
       doubleSlash = 1;
-      strcpy(line, &line[1]);
+      line++;
+    }
+    if(line[0] != '/') {
+      if(!m_log_req_regex) {
+	m_log_req_regex = 1;
+	fprintf(stderr, "WARNING, line %d: "
+		"unexpected data format, try to detect request lines automatically\n",
+		line_nr);
+      }
+      qos_auto_detect(&line);
     }
     if(apr_uri_parse(lpool, line, &parsed_uri) != APR_SUCCESS) {
       fprintf(stderr, "ERROR, could parse uri %s\n", line);
@@ -1219,6 +1327,8 @@ static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
     }
     if(parsed_uri.path == NULL || (parsed_uri.path[0] != '/')) {
       fprintf(stderr, "WARNING, line %d: invalid request %s\n", line_nr, line);
+    } else if(m_filter && parsed_uri.path && strncmp(parsed_uri.path, m_filter, strlen(m_filter)) != 0) {
+      // skip filtered line
     } else {
       char *path = NULL;
       char *query = NULL;
@@ -1372,16 +1482,20 @@ static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
 }
 
 static void qos_measurement(apr_pool_t *pool, apr_table_t *blacklist, apr_table_t *rules, FILE *f, int *ln) {
-  char *line = apr_pcalloc(pool, MAX_LINE_BUFFER);
+  char *readline = apr_pcalloc(pool, MAX_LINE_BUFFER);
   int line_nr = 0;
-  while(!qos_fgetline(line, MAX_LINE_BUFFER, f)) {
+  while(!qos_fgetline(readline, MAX_LINE_BUFFER, f)) {
     apr_uri_t parsed_uri;
     apr_pool_t *lpool;
+    char *line = readline;
     apr_pool_create(&lpool, NULL);
     line_nr++;
     if((strlen(line) > 1) && line[1] == '/') {
       strcpy(line, &line[1]);
     }
+    if(line[0] != '/') {
+      qos_auto_detect(&line);
+    }
     if(apr_uri_parse(lpool, line, &parsed_uri) != APR_SUCCESS) {
       fprintf(stderr, "ERROR, could parse uri %s\n", line);
       if(m_exit_on_error) exit(1);
@@ -1451,6 +1565,14 @@ int main(int argc, const char * const argv[]) {
       if (--argc >= 1) {
 	access_log = *(++argv);
       }
+    } else if(strcmp(*argv,"-k") == 0) {
+      if (--argc >= 1) {
+	m_pfx = *(++argv);
+      }
+    } else if(strcmp(*argv,"-f") == 0) {
+      if (--argc >= 1) {
+	m_filter = *(++argv);
+      }
     } else if(strcmp(*argv,"-d") == 0) {
       if (--argc >= 1) {
 	m_path_depth = atoi(*(++argv));
@@ -1535,7 +1657,7 @@ int main(int argc, const char * const argv[]) {
     qos_delete_obsolete_rules(pool, rules, rules_url);
     // ensure, we have not deleted to many!
     if(m_verbose) {
-      printf("# verfiy new rules ...\n"); 
+      printf("# verify new rules ...\n"); 
       fflush(stdout);
     }
     //    if(httpdconf) {
@@ -1572,7 +1694,7 @@ int main(int argc, const char * const argv[]) {
   printf("#  path depth (-d): %d\n", m_path_depth);
   printf("#  disable path only regex (-h): %s\n", m_handler == 1 ? "yes" : "no");
   printf("#  base64 detection level (-b): %d\n", m_base64);
-  printf("#  redundancy check (-n): %s\n", m_redundant == 1 ? "on" : "off");
+  printf("#  redundancy check (-n): %s\n", m_redundant == 1 ? "yes" : "no");
   printf("#  pcre only for query (-p): %s\n", m_query_pcre == 1 ? "yes" : "no");
   printf("#  decoding (-u): url");
   if(m_mode & QOS_DEC_MODE_FLAGS_UNI) {
@@ -1602,7 +1724,7 @@ int main(int argc, const char * const argv[]) {
   printf("# --------------------------------------------------------\n");
 
   {
-    STACK *st = sk_new(STACK_qs_cmp);
+    STACK_OF(qs_rule_t) *st = sk_new(STACK_qs_cmp);
     qs_rule_t *r;
     int j = 1;
     entry = (apr_table_entry_t *)apr_table_elts(rules)->elts;
@@ -1616,7 +1738,9 @@ int main(int argc, const char * const argv[]) {
     i = sk_num(st);
     for(; i > 0; i--) {
       r = (qs_rule_t *)sk_value(st, i-1);
-      printf("QS_PermitUri +QSF%.3d deny \"%s\"\n", j, r->rule);
+      printf("QS_PermitUri +%s%.3d deny \"%s\"\n",
+	     m_pfx ? m_pfx : "QSF",
+	     j, qs_apache_escape(pool, r->rule));
       j++;
     }
   }
