@@ -3,10 +3,10 @@
  *
  * Real time access log data correlation.
  *
- * See http://sourceforge.net/projects/mod-qos/ for further
+ * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2010 Pascal Buchbinder
+ * Copyright (C) 2007-2011 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.1 2010/08/17 18:32:34 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.12 2011/03/29 17:56:50 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -41,7 +41,7 @@ static const char revision[] = "$Id: qslog.c,v 1.1 2010/08/17 18:32:34 pbuchbind
 #include <regex.h>
 #include <time.h>
 
-#include <config.h>
+//#include <config.h>
 
 #include "qs_util.h"
 
@@ -59,6 +59,7 @@ static const char revision[] = "$Id: qslog.c,v 1.1 2010/08/17 18:32:34 pbuchbind
  * global stat counter
  * ---------------------------------- */
 static long m_line_count = 0;
+static long long m_i_byte_count = -1;
 static long long m_byte_count = 0;
 static long m_duration_count = 0;
 static long m_duration_0 = 0;
@@ -69,12 +70,19 @@ static long m_duration_4 = 0;
 static long m_duration_5 = 0;
 static long m_duration_6 = 0;
 
+static long m_status_1 = 0;
+static long m_status_2 = 0;
+static long m_status_3 = 0;
+static long m_status_4 = 0;
+static long m_status_5 = 0;
+
 static long m_qos_v = 0;
 static long m_qos_s = 0;
 static long m_qos_d = 0;
 static long m_qos_k = 0;
 static long m_qos_t = 0;
 static long m_qos_l = 0;
+static long m_qos_ser = 0;
 
 static qs_event_t *m_ip_list = NULL;
 static qs_event_t *m_user_list = NULL;
@@ -131,7 +139,9 @@ static char *cutNext(char **line) {
   char *c = *line;
   char *p = skipElement(*line);
   *line = p;
-  p--; p[0] = '\0';
+  if(p[0]) {
+    p--; p[0] = '\0';
+  }
   /* cut leading and tailing " */
   if(c[0] == '\"') {
     c++;
@@ -217,6 +227,7 @@ static void getFreeMem(char *buf, int sz) {
 /* value names in csv output */
 #define NRS "r/s"
 #define NBS "b/s"
+#define NBIS "ib/s"
 #define NAV "av"
 
 /*
@@ -226,6 +237,11 @@ static void getFreeMem(char *buf, int sz) {
 static void printAndResetStat(char *timeStr) {
   double av[1];
   char mem[256];
+  char bis[256];
+  bis[0] = '\0';
+  if(m_i_byte_count != -1) {
+    sprintf(bis, NBIS";%lld;", m_i_byte_count/LOG_INTERVAL);
+  }
   if(!m_offline) {
     getloadavg(av, 1);
     if(m_mem) {
@@ -241,6 +257,12 @@ static void printAndResetStat(char *timeStr) {
           NRS";%ld;"
 	  "req;%ld;"
           NBS";%lld;"
+	  "%s"
+	  "1xx;%ld;"
+	  "2xx;%ld;"
+	  "3xx;%ld;"
+	  "4xx;%ld;"
+	  "5xx;%ld;"
           NAV";%ld;"
           "<1s;%ld;"
           "1s;%ld;"
@@ -251,17 +273,24 @@ static void printAndResetStat(char *timeStr) {
           ">5s;%ld;"
 	  "ip;%ld;"
 	  "usr;%ld;"
-	  "qv;%ld;"
+	  "qV;%ld;"
+	  "qS;%ld;"
+	  "qD;%ld;"
+	  "qK;%ld;"
+	  "qT;%ld;"
+	  "qL;%ld;"
 	  "qs;%ld;"
-	  "qd;%ld;"
-	  "qk;%ld;"
-	  "qt;%ld;"
-	  "ql;%ld;"
 	  ,
 	  timeStr,
           m_line_count/LOG_INTERVAL,
 	  m_line_count,
           m_byte_count/LOG_INTERVAL,
+	  bis,
+	  m_status_1,
+	  m_status_2,
+	  m_status_3,
+	  m_status_4,
+	  m_status_5,
           m_duration_count/(m_line_count == 0 ? 1 : m_line_count),
           m_duration_0,
           m_duration_1,
@@ -277,10 +306,19 @@ static void printAndResetStat(char *timeStr) {
 	  m_qos_d,
 	  m_qos_k,
 	  m_qos_t,
-	  m_qos_l
+	  m_qos_l,
+	  m_qos_ser
           );
   m_line_count = 0;
   m_byte_count = 0;
+  if(m_i_byte_count != -1) {
+    m_i_byte_count = 0;
+  }
+  m_status_1 = 0;
+  m_status_2 = 0;
+  m_status_3 = 0;
+  m_status_4 = 0;
+  m_status_5 = 0;
   m_duration_count = 0;
   m_duration_0 = 0;
   m_duration_1 = 0;
@@ -295,6 +333,7 @@ static void printAndResetStat(char *timeStr) {
   m_qos_k = 0;
   m_qos_t = 0;
   m_qos_l = 0;
+  m_qos_ser = 0;
   if(!m_offline) {
     fprintf(m_f, "sl;%.2f;m;%s",
 	    av[0], mem[0] ? mem : "-");
@@ -320,6 +359,8 @@ static void printAndResetStat(char *timeStr) {
  */
 static void updateStat(const char *cstr, char *line) {
   char *T = NULL; /* time */
+  char *S = NULL; /* status */
+  char *BI = NULL; /* bytes in */
   char *B = NULL; /* bytes */
   char *R = NULL; /* request line */
   char *I = NULL; /* client ip */
@@ -330,35 +371,43 @@ static void updateStat(const char *cstr, char *line) {
   if(!line[0]) return;
   while(c[0]) {
     /* process known types */
-    if(strncmp(c, ".", 1) == 0) {
+    if(c[0] == '.') {
       if(l != NULL && l[0] != '\0') {
         l = skipElement(l);
       }
-    } else if(strncmp(c, "T", 1) == 0) {
+    } else if(c[0] == 'T') {
       if(l != NULL && l[0] != '\0') {
         T = cutNext(&l);
       }
-    } else if(strncmp(c, "B", 1) == 0) {
+    } else if(c[0] == 'S') {
+      if(l != NULL && l[0] != '\0') {
+        S = cutNext(&l);
+      }
+    } else if(c[0] == 'B') {
       if(l != NULL && l[0] != '\0') {
         B = cutNext(&l);
       }
-    } else if(strncmp(c, "R", 1) == 0) {
+    } else if(c[0] == 'i') {
+      if(l != NULL && l[0] != '\0') {
+        BI = cutNext(&l);
+      }
+    } else if(c[0] == 'R') {
       if(l != NULL && l[0] != '\0') {
         R = cutNext(&l);
       }
-    } else if(strncmp(c, "I", 1) == 0) {
+    } else if(c[0] == 'I') {
       if(l != NULL && l[0] != '\0') {
         I = cutNext(&l);
       }
-    } else if(strncmp(c, "U", 1) == 0) {
+    } else if(c[0] == 'U') {
       if(l != NULL && l[0] != '\0') {
         U = cutNext(&l);
       }
-    } else if(strncmp(c, "Q", 1) == 0) {
+    } else if(c[0] == 'Q') {
       if(l != NULL && l[0] != '\0') {
         Q = cutNext(&l);
       }
-    } else if(strncmp(c, " ", 1) == 0) {
+    } else if(c[0] == ' ') {
       /* do nothing */
     } else {
       /* undedined char, skip it */
@@ -370,10 +419,12 @@ static void updateStat(const char *cstr, char *line) {
   }
   if(m_offline && m_verbose) {
     m_lines++;
-    printf("[%ld] I=%s U=%s B=%s T=%s Q=%s\n", m_lines,
+    printf("[%ld] I=%s U=%s B=%s i=%s S=%s T=%s Q=%s\n", m_lines,
 	   I == NULL ? "(null)" : I,
 	   U == NULL ? "(null)" : U,
 	   B == NULL ? "(null)" : B,
+	   BI == NULL ? "(null)" : BI,
+	   S == NULL ? "(null)" : S,
 	   T == NULL ? "(null)" : T,
 	   Q == NULL ? "(null)" : Q
 	   );
@@ -399,6 +450,9 @@ static void updateStat(const char *cstr, char *line) {
     if(strchr(Q, 'L') != NULL) {
       m_qos_l++;
     }
+    if(strchr(Q, 's') != NULL) {
+      m_qos_ser++;
+    }
   }
   if(I != NULL) {
     /* update/store client IP */
@@ -412,6 +466,25 @@ static void updateStat(const char *cstr, char *line) {
     /* transferred bytes */
     m_byte_count = m_byte_count + atoi(B);
   }
+  if(BI != NULL) {
+    /* transferred bytes */
+    m_i_byte_count = m_i_byte_count + atoi(BI);
+  }
+  if(S != NULL) {
+    if(S[0] == '1') {
+      m_status_1++;
+    } else if(S[0] == '1') {
+      m_status_1++;
+    } else if(S[0] == '2') {
+      m_status_2++;
+    } else if(S[0] == '3') {
+      m_status_3++;
+    } else if(S[0] == '4') {
+      m_status_4++;
+    } else {
+      m_status_5++;
+    }
+  }
   if(T != NULL) {
     /* response duration */
     int tme = atoi(T);
@@ -582,6 +655,7 @@ static void *loggerThread(void *argv) {
       }
     }
   }
+  return NULL;
 }
 
 static void usage(char *cmd) {
@@ -595,7 +669,9 @@ static void usage(char *cmd) {
   printf("the data from stdin. The output is written to the specified\n");
   printf("file every minute. The output includes the following entries:\n");
   printf("  - requests per second ("NRS")\n");
-  printf("  - bytes (http body data) sent to the client per second ("NBS")\n");
+  printf("  - bytes sent to the client per second ("NBS")\n");
+  printf("  - bytes received from the client per second ("NBIS")\n");
+  printf("  - repsonse status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
   printf("  - average response duration ("NAV")\n");
   printf("  - distribution of response durations within the last minute\n");
   printf("    (<1s,1s,2s,3s,4s,5s,>5)\n");
@@ -603,9 +679,9 @@ static void usage(char *cmd) {
   printf("  - free memory (m) (not available for all platforms)\n");
   printf("  - number of client ip addresses seen withn the last %d seconds (ip)\n", ACTIVE_TIME);
   printf("  - number of different users seen withn the last %d seconds (usr)\n", ACTIVE_TIME);
-  printf("  - number of mod_qos events within the last minute (qv=create session,\n");
-  printf("    qs=session pass, qd=access denied, qk=connection closed, qt=dynamic\n");
-  printf("    keep-alive, ql=request/response slow down)\n");
+  printf("  - number of mod_qos events within the last minute (qV=create session,\n");
+  printf("    qS=session pass, qD=access denied, qK=connection closed, qT=dynamic\n");
+  printf("    keep-alive, qL=request/response slow down, qs=serialized request)\n");
   printf("\n");
   printf("Options\n");
   printf("  -f <format_string>\n");
@@ -614,10 +690,12 @@ static void usage(char *cmd) {
   printf("     See to the 'LogFormat' directive of the httpd.conf file\n");
   printf("     to see the format defintions of the servers access log\n");
   printf("     data. %s knows the following elements:\n", cmd);
-  printf("     T defines the request duration (%%T)\n");
-  printf("     B defines the transferred bytes (%%b)\n");
-  printf("     R defines the request line (%%r)\n");
   printf("     I defines the client ip address (%%h)\n");
+  printf("     R defines the request line (%%r)\n");
+  printf("     S defines HTTP response status code (%%s)\n");
+  printf("     B defines the transferred bytes (%%b or %%O)\n");
+  printf("     i defines the received bytes (%%I)\n");
+  printf("     T defines the request duration (%%T)\n");
   printf("     U defines the user tracking id (%%{mod_qos_user_id}e)\n");
   printf("     Q defines the mod_qos_ev event message (%%{mod_qos_ev}o)\n");
   printf("     . defines an element to ignore (unknown string)\n");
@@ -638,12 +716,15 @@ static void usage(char *cmd) {
   printf("\n");
   printf("Example configuration using pipped logging:\n");
   printf("  LogFormat \"%%t %%h \\\"%%r\\\" %%>s %%b \\\"%%{User-Agent}i\\\" %%T\"\n");
-  printf("  TransferLog \"|./bin/%s -f ..IR.B.T -o ./logs/stat_log\"\n", cmd);
+  printf("  TransferLog \"|./bin/%s -f ..IRSB.T -o ./logs/stat_log\"\n", cmd);
+  printf("\n");
+  printf("Example configuration using the CustomLog directive:\n");
+  printf("  CustomLog \"|./bin/%s -f ISBT -o ./logs/stat_log\" \"%%h %%>s %%b %%T\"\n", cmd);
   printf("\n");
   printf("Example for post processing:\n");
-  printf("  cat access_log | ./bin/%s -f ..IR.B.T -o ./logs/stat_log -p\n", cmd);
+  printf("  cat access_log | ./bin/%s -f ..IRSB.T -o ./logs/stat_log -p\n", cmd);
   printf("\n");
-  printf("See http://mod-qos.sourceforge.net/ for further details.\n");
+  printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
   exit(1);
 }
 
@@ -667,6 +748,10 @@ int main(int argc, char **argv) {
     if(strcmp(*argv,"-f") == 0) { /* this is the format string */
       if (--argc >= 1) {
 	config = *(++argv);
+	if(strchr(config, 'i')) {
+	  // enable ib/s
+	  m_i_byte_count = 0;
+	}
       }
     } else if(strcmp(*argv,"-o") == 0) { /* this is the out file */
       if (--argc >= 1) {
