@@ -1,10 +1,10 @@
 /**
  * Utilities for the quality of service module mod_qos.
  *
- * See http://sourceforge.net/projects/mod-qos/ for further
+ * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2010 Pascal Buchbinder
+ * Copyright (C) 2007-2011 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@
  *
  */
 
-static const char revision[] = "$Id: qspng.c,v 1.1 2010/08/17 18:32:34 pbuchbinder Exp $";
+static const char revision[] = "$Id: qspng.c,v 1.7 2011/03/04 16:56:14 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <strings.h>
@@ -32,7 +32,7 @@ static const char revision[] = "$Id: qspng.c,v 1.1 2010/08/17 18:32:34 pbuchbind
 #include <unistd.h>
 #include <png.h>
 
-#include <config.h>
+//#include <config.h>
 
 #include "qs_util.h"
 #include "char.h"
@@ -58,7 +58,8 @@ typedef struct {
 static const qs_png_elt_t qs_png_elts[] = {
   { "r/s", "requests per second",  20, 30, 130, },
   { "req", "requests per minute",  20, 30, 130, },
-  { "b/s", "bytes per second",     30, 45, 130 },
+  { "b/s", "bytes per second (out)",     30, 45, 130 },
+  { "ib/s", "bytes per second (in)",     30, 45, 125 },
   { "av", "average response time", 40, 95, 140 },
   { "<1s", "requests faster than 1 second", 35, 95, 180 },
   { "1s", "requests faster or equal than 1 second", 35, 90, 180 },
@@ -122,7 +123,7 @@ void lp_write_data(png_structp png_ptr, png_bytep data, png_size_t length) {
 
 /* png io callback (not used) */
 void lp_flush_data(png_structp png_ptr) {
-  FILE *f = png_get_io_ptr(png_ptr);
+  png_get_io_ptr(png_ptr);
   fprintf(stderr, "flush\n");
 }
 
@@ -344,7 +345,7 @@ static void qs_png_label(int width, int height, int border,
                         
 }
 
-lp_init(int width, int height, int border, png_bytep **start) {
+static void lp_init(int width, int height, int border, png_bytep **start) {
   png_bytep *row_pointers;
   int b_width = width + (2 * border);
   int b_height = height + (2 * border);
@@ -562,19 +563,18 @@ static void usage(char *cmd) {
   printf("  -o <out_file>\n");
   printf("     Output file name, e.g. stat.png.\n");
   printf("\n");
-  printf("See http://mod-qos.sourceforge.net/ for further details.\n");
+  printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
   exit(1);
 }
 
 int main(int argc, char **argv) {
-  int x, y;
+  int y;
   int width, height, b_width, b_height;
   png_byte color_type;
   png_byte bit_depth;
 
   int scale;
 
-  char nameBuffer[1024];
   png_structp png_ptr;
   png_infop info_ptr;
 
@@ -592,7 +592,6 @@ int main(int argc, char **argv) {
   int c_g = 50;
   int c_b = 175;
   const qs_png_elt_t* elt;
-  int i, j;
 
   if(cmd == NULL) {
     cmd = argv[0];
@@ -649,7 +648,7 @@ int main(int argc, char **argv) {
   }
   info_ptr = png_create_info_struct(png_ptr);
   if(info_ptr == NULL) {
-    fprintf(stderr,"[%s]: ERROR, could not create png infromation struct\n", cmd);
+    fprintf(stderr,"[%s]: ERROR, could not create png information struct\n", cmd);
     exit(1);
   }
   if(setjmp(png_jmpbuf(png_ptr))) {
