@@ -4,10 +4,10 @@
  *
  * Log rotation tool.
  *
- * See http://sourceforge.net/projects/mod-qos/ for further
+ * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2010 Pascal Buchbinder
+ * Copyright (C) 2007-2011 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,7 +26,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsrotate.c,v 1.1 2010/08/17 18:32:34 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsrotate.c,v 1.5 2011/01/01 20:52:04 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -49,7 +49,7 @@ static const char revision[] = "$Id: qsrotate.c,v 1.1 2010/08/17 18:32:34 pbuchb
 #include <sys/wait.h>
 #include <sys/stat.h>
 
-#include <config.h>
+//#include <config.h>
 
 #include "qs_util.h"
 
@@ -100,7 +100,7 @@ static void usage(char *cmd) {
   printf("Note\n");
   printf("  - Each %s instance must use an individual file!\n", cmd);
   printf("\n");
-  printf("See http://mod-qos.sourceforge.net/ for further details.\n");
+  printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
   exit(1);
 }
 
@@ -258,6 +258,7 @@ static void *forcedRotationThread(void *argv) {
     n = 1 + m_tLogEnd - now;
     sleep(n);
   }
+  return NULL;
 }
 
 int main(int argc, char **argv) {
